/*
 *   ScenarLang a domain specific language that attempts to make it even easier
 *   to write unit tests with mockito
 *    
 *   Copyright (C) 2012  X. Gillard 
 *   
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *   
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *   
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */
package xgi.ut.dsl.interpreters

import scala.collection.JavaConversions._
import xgi.ut.dsl.ObjectParsers._
import xgi.ut.dsl.ScenarParsers._
import xgi.ut.dsl.utl.ReflectionUtil._
import xgi.ut.dsl.utl.ReflectionWrapper._
import scala.collection.mutable.HashMap

/**
 * This trait provides the capability of interpreting a ScenarLang script.
 */
trait ScenarioInterpreter extends ToObject  {
  this:{val context:Any} =>
  /* this is where we store the variables being defined in the scenario */
  val variables = HashMap.empty[String, Any]
  
  /** interprets one instruction */
  def interpret(i:Instruction):Any = i match {
    case SyntaxError(msg) => Console.err.println(msg); throw new RuntimeException(msg)
    case ConfigInstruction(config) => addConfig(config)
    case Declaration(name, value) => {
      val valueObject = toObject(value)
      variables += name->valueObject
      valueObject
    }
    case Assignment(destination, value) => {
      val valueObject = toObject(value)
      val source = dereference(destination.source, context)
      val interpretation = (source /: destination.invocations.dropRight(1))((newctx, elem) => dereference(elem, newctx))
      assignResult(interpretation, destination.invocations.last, valueObject)
      valueObject
    }
  }
  
  /** assigns a mock result to a given function invocation */
  private[this] def assignResult(target:Any, ref:Reference, result:Any = null):Any = ref match {
    case MemberRef(src, destinations) => {
      val source = dereference(src, if(target!=null)target else context)
      val newTarget = (source /: destinations.dropRight(1))((newctx, elem) => dereference(elem, newctx))
      assignResult(newTarget, destinations.last, result)
    }
    // an internal would make no sense here
    case FieldRef(name) => {
	  setField(target, name, result)
	  getFieldValue(target, name)
    }
    case FnInvocation(fname, params) => {
      val args = params map{toObject}
      invoke(target, fname, args, result)
    }
  }
}
